﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Hilbert
{
    public class HilbertCurve
    {
        /// <summary>
        /// Given an origin, the x-axis, y-axis, and the order of the curve
        /// Returns a list of points in 2D along the hilbert curve.
        /// The length of the returned list is 2^order.
        /// </summary>
        /// <param name="origin"></param>
        /// <param name="xAxis"></param>
        /// <param name="yAxis"></param>
        /// <param name="order"></param>
        /// <returns></returns>
        public static List< Vector2f > GenerateFloat2( Vector2f origin,
            Vector2f xAxis, Vector2f yAxis, int order )
        {
            int sideLength = ( 1 << order );
            int capacity = sideLength * sideLength;
            var output = new List< Vector2f >( capacity );
            HilbertFloat( origin.X, origin.Y, xAxis.X, xAxis.Y, yAxis.X, yAxis.Y, order, output );
            return output;
        }

        private static void HilbertFloat( float x0, float y0,
            float xi, float xj,
            float yi, float yj,
            int n,
            List< Vector2f > output )
        {
            if( n <= 0 )
            {
                float x = x0 + ( xi + yi ) / 2;
                float y = y0 + ( xj + yj ) / 2;
                
                output.Add( new Vector2f( x, y ) );
            }
            else
            {
                HilbertFloat( x0, y0, yi / 2, yj / 2, xi / 2, xj / 2, n - 1, output );
                HilbertFloat( x0 + xi / 2, y0 + xj / 2, xi / 2, xj / 2, yi / 2, yj / 2, n - 1, output );
                HilbertFloat( x0 + xi / 2 + yi / 2, y0 + xj / 2 + yj / 2, xi / 2, xj / 2, yi / 2, yj / 2, n - 1, output );
                HilbertFloat( x0 + xi / 2 + yi, y0 + xj / 2 + yj, -yi / 2, -yj / 2, -xi / 2, -xj / 2, n - 1, output );
            }
        }
    }
}
